@@ -26,10 +26,13 @@ context or provide additional information in their specific WSGI pipeline
 or logging context.
 """
 
-import inspect
+import collections
 import itertools
 import threading
 import uuid
+import warnings
+
+from positional import positional
 
 
 _request_store = threading.local()
@@ -59,6 +62,62 @@ def generate_request_id():
     return 'req-%s' % uuid.uuid4()
 
 
+class _DeprecatedPolicyValues(collections.MutableMapping):
+    """A Dictionary that manages current and deprecated policy values.
+
+    Anything added to this dictionary after initial creation is considered a
+    deprecated key that we are trying to move services away from. Accessing
+    these values as oslo.policy will do will trigger a DeprecationWarning.
+    """
+
+    def __init__(self, data):
+        self._data = data
+        self._deprecated = {}
+
+    def __getitem__(self, k):
+        try:
+            return self._data[k]
+        except KeyError:
+            pass
+
+        try:
+            val = self._deprecated[k]
+        except KeyError:
+            pass
+        else:
+            warnings.warn('Policy enforcement is depending on the value of '
+                          '%s. This key is deprecated. Please update your '
+                          'policy file to use the standard policy values.' % k,
+                          DeprecationWarning)
+            return val
+
+        raise KeyError(k)
+
+    def __setitem__(self, k, v):
+        self._deprecated[k] = v
+
+    def __delitem__(self, k):
+        del self._deprecated[k]
+
+    def __iter__(self):
+        return iter(self._dict)
+
+    def __len__(self):
+        return len(self._dict)
+
+    def __str__(self):
+        return self._dict.__str__()
+
+    def __repr__(self):
+        return self._dict.__repr__()
+
+    @property
+    def _dict(self):
+        d = self._deprecated.copy()
+        d.update(self._data)
+        return d
+
+
 class RequestContext(object):
 
     """Helper class to represent useful information about a request context.
@@ -69,6 +128,7 @@ class RequestContext(object):
 
     user_idt_format = u'{user} {tenant} {domain} {user_domain} {p_domain}'
 
+    @positional(enforcement=positional.WARN)
     def __init__(self, auth_token=None, user=None, tenant=None, domain=None,
                  user_domain=None, project_domain=None, is_admin=False,
                  read_only=False, show_deleted=False, request_id=None,
@@ -126,12 +186,17 @@ class RequestContext(object):
         with either deprecated values or additional attributes used by that
         service specific policy.
         """
-        return {'user_id': self.user,
-                'user_domain_id': self.user_domain,
-                'project_id': self.tenant,
-                'project_domain_id': self.project_domain,
-                'roles': self.roles,
-                'is_admin_project': self.is_admin_project}
+        # NOTE(jamielennox): We need a way to allow projects to provide old
+        # deprecated policy values that trigger a warning when used in favour
+        # of our standard ones. This object acts like a dict but only values
+        # from oslo.policy don't show a warning.
+        return _DeprecatedPolicyValues({
+            'user_id': self.user,
+            'user_domain_id': self.user_domain,
+            'project_id': self.tenant,
+            'project_domain_id': self.project_domain,
+            'roles': self.roles,
+            'is_admin_project': self.is_admin_project})
 
     def to_dict(self):
         """Return a dictionary of context attributes."""
@@ -168,12 +233,28 @@ class RequestContext(object):
         return values
 
     @classmethod
-    def from_dict(cls, values):
+    def from_dict(cls, values, **kwargs):
         """Construct a context object from a provided dictionary."""
-        allowed = [arg for arg in
-                   inspect.getargspec(RequestContext.__init__).args
-                   if arg != 'self']
-        kwargs = {k: v for (k, v) in values.items() if k in allowed}
+        kwargs.setdefault('auth_token', values.get('auth_token'))
+        kwargs.setdefault('user', values.get('user'))
+        kwargs.setdefault('tenant', values.get('tenant'))
+        kwargs.setdefault('domain', values.get('domain'))
+        kwargs.setdefault('user_domain', values.get('user_domain'))
+        kwargs.setdefault('project_domain', values.get('project_domain'))
+        kwargs.setdefault('is_admin', values.get('is_admin', False))
+        kwargs.setdefault('read_only', values.get('read_only', False))
+        kwargs.setdefault('show_deleted', values.get('show_deleted', False))
+        kwargs.setdefault('request_id', values.get('request_id'))
+        kwargs.setdefault('resource_uuid', values.get('resource_uuid'))
+        kwargs.setdefault('roles', values.get('roles'))
+        kwargs.setdefault('user_name', values.get('user_name'))
+        kwargs.setdefault('project_name', values.get('project_name'))
+        kwargs.setdefault('domain_name', values.get('domain_name'))
+        kwargs.setdefault('user_domain_name', values.get('user_domain_name'))
+        kwargs.setdefault('project_domain_name',
+                          values.get('project_domain_name'))
+        kwargs.setdefault('is_admin_project',
+                          values.get('is_admin_project', True))
         return cls(**kwargs)
 
     @classmethod
