@@ -14,8 +14,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import fixtures
 import hashlib
 import uuid
+import warnings
 
 from oslotest import base as test_base
 
@@ -27,6 +29,21 @@ def generate_id(name):
     return hashlib.md5(name.encode('utf-8')).hexdigest()
 
 
+class WarningsFixture(fixtures.Fixture):
+
+    def __init__(self, action="always", category=DeprecationWarning):
+        super(WarningsFixture, self).__init__()
+        self.action = action
+        self.category = category
+
+    def setUp(self):
+        super(WarningsFixture, self).setUp()
+        self._w = warnings.catch_warnings(record=True)
+        self.log = self._w.__enter__()
+        self.addCleanup(self._w.__exit__)
+        warnings.simplefilter(self.action, self.category)
+
+
 class Object(object):
     pass
 
@@ -35,6 +52,7 @@ class ContextTest(test_base.BaseTestCase):
 
     def setUp(self):
         super(ContextTest, self).setUp()
+        self.warnings = self.useFixture(WarningsFixture())
         self.useFixture(fixture.ClearRequestContext())
 
     def test_context(self):
@@ -137,6 +155,25 @@ class ContextTest(test_base.BaseTestCase):
         self.assertTrue(ctx.read_only)
         self.assertRaises(KeyError, lambda: ctx.__dict__['color'])
 
+    def test_from_dict_overrides(self):
+        dct = {
+            "auth_token": "token1",
+            "user": "user1",
+            "read_only": True,
+            "roles": "role1,role2,role3",
+            "color": "red",
+            "unknown": ""
+        }
+        ctx = context.RequestContext.from_dict(dct,
+                                               user="user2",
+                                               project_name="project1")
+        self.assertEqual("token1", ctx.auth_token)
+        self.assertEqual("user2", ctx.user)
+        self.assertEqual("project1", ctx.project_name)
+        self.assertIsNone(ctx.tenant)
+        self.assertFalse(ctx.is_admin)
+        self.assertTrue(ctx.read_only)
+
     def test_is_user_context(self):
         self.assertFalse(context.is_user_context(None))
         ctx = context.RequestContext(is_admin=True)
@@ -227,27 +264,27 @@ class ContextTest(test_base.BaseTestCase):
                    'HTTP_X_USER_ID': new}
 
         ctx = context.RequestContext.from_environ(environ=environ)
-        self.assertEqual(ctx.user, new)
+        self.assertEqual(new, ctx.user)
 
         ctx = context.RequestContext.from_environ(environ=environ,
                                                   user=override)
-        self.assertEqual(ctx.user, override)
+        self.assertEqual(override, ctx.user)
 
         environ = {'HTTP_X_TENANT': old,
                    'HTTP_X_PROJECT_ID': new}
 
         ctx = context.RequestContext.from_environ(environ=environ)
-        self.assertEqual(ctx.tenant, new)
+        self.assertEqual(new, ctx.tenant)
 
         ctx = context.RequestContext.from_environ(environ=environ,
                                                   tenant=override)
-        self.assertEqual(ctx.tenant, override)
+        self.assertEqual(override, ctx.tenant)
 
         environ = {'HTTP_X_TENANT_NAME': old,
                    'HTTP_X_PROJECT_NAME': new}
 
         ctx = context.RequestContext.from_environ(environ=environ)
-        self.assertEqual(ctx.project_name, new)
+        self.assertEqual(new, ctx.project_name)
 
     def test_from_environ_strip_roles(self):
         environ = {'HTTP_X_ROLES': ' abc\t,\ndef\n,ghi\n\n'}
@@ -434,7 +471,6 @@ class ContextTest(test_base.BaseTestCase):
                           'is_admin_project': True},
                          ctx.to_policy_values())
 
-        # is_admin_project False gets passed through
         ctx = context.RequestContext(user=user,
                                      user_domain=user_domain,
                                      tenant=tenant,
@@ -449,3 +485,39 @@ class ContextTest(test_base.BaseTestCase):
                           'roles': roles,
                           'is_admin_project': False},
                          ctx.to_policy_values())
+
+    def test_positional_args(self):
+        context.RequestContext('abc', 'def')
+
+        self.assertEqual(1, len(self.warnings.log))
+        self.assertIn('__init__ takes at most 1 positional',
+                      str(self.warnings.log[0].message))
+
+    def test_policy_deprecations(self):
+        user = uuid.uuid4().hex
+        user_domain = uuid.uuid4().hex
+        tenant = uuid.uuid4().hex
+        project_domain = uuid.uuid4().hex
+        roles = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
+
+        ctx = context.RequestContext(user=user,
+                                     user_domain=user_domain,
+                                     tenant=tenant,
+                                     project_domain=project_domain,
+                                     roles=roles)
+
+        policy = ctx.to_policy_values()
+        key = uuid.uuid4().hex
+        val = uuid.uuid4().hex
+
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+
+            # no warning triggered by adding key to dict
+            policy[key] = val
+            self.assertEqual(0, len(w))
+
+            # warning triggered by fetching key from dict
+            self.assertIs(val, policy[key])
+            self.assertEqual(1, len(w))
+            self.assertIn(key, str(w[0].message))
